@@ -4,6 +4,7 @@ from .visualization import *
 from .datasets import OpticalSatellite, ImageCollection
 from .bands import OpticalBand, BitBand, ClassificationBand, ExpressionBand
 from .masks import Mask
+from . import helpers
 import geetools
 from . import register
 import ee
@@ -69,6 +70,29 @@ Visualizers: {visualizers}
 
     extra_bands = (ndvi, nbr, ndre)
 
+    def add_cloud_probability(self, collection=None, name='cloud_probability'):
+        """ add band from ee.ImageCollection("COPERNICUS/S2_CLOUD_PROBABILITY")
+        to the specified collection. If collection is None the complete S2
+        collection will be used. You can use any collection as long as it keeps
+        the image ids """
+        if collection is None:
+            collection = self.collection()
+
+        clouds = ee.ImageCollection("COPERNICUS/S2_CLOUD_PROBABILITY")
+        propertyField = 'system:index'
+        Filter = ee.Filter.equals(leftField=propertyField,
+                                  rightField=propertyField)
+        join = ee.Join.saveFirst(matchKey='match', outer=False)
+        joined = ee.ImageCollection(join.apply(collection, clouds, Filter))
+
+        def overJoined(image):
+            # properties = image.propertyNames()
+            # retain = properties.remove('match')
+            match = ee.Image(image.get('match')).rename(name)
+            return image.addBands(match)
+
+        return joined.map(overJoined)
+
     def __init__(self, **kwargs):
         super(Sentinel2, self).__init__(**kwargs)
 
@@ -109,9 +133,34 @@ class Sentinel2TOA(Sentinel2):
 
     masks = (Mask.fromBand('QA60', Sentinel2.qa60),
              Mask('Hollstein',
-                  negatives= ('cloud', 'snow', 'shadow', 'water', 'cirrus'),
+                  negatives=('cloud', 'snow', 'shadow', 'water', 'cirrus'),
                   decoder=decode_hollstein))
 
+    def add_scl(self, collection=None, name='SCL', renamed=False):
+        """ Add the SCL band from S2SR collection to every image of the
+        given collection. If collection is None it will use the complete S2TOA
+        collection """
+        s2sr = Sentinel2SR()
+        S2SR_start = s2sr.start_date
+        if collection is None:
+            collection = self.collection().filterDate(S2SR_start, helpers.today())
+        srcol = s2sr.collection()
+        propertyField = 'system:index'
+        Filter = ee.Filter.equals(leftField=propertyField,
+                                  rightField=propertyField)
+        join = ee.Join.saveFirst(matchKey='match', outer=False)
+        joined = ee.ImageCollection(join.apply(collection, srcol, Filter))
+
+        def overJoined(image):
+            # properties = image.propertyNames()
+            # retain = properties.remove('match')
+            match = ee.Image(image.get('match'))\
+                      .select(s2sr.scl.alias if renamed else s2sr.scl.name)\
+                      .rename(name)
+            return image.addBands(match)
+
+        return joined.map(overJoined)
+
     def __init__(self, **kwargs):
         super(Sentinel2TOA, self).__init__(**kwargs)
 
@@ -206,7 +255,7 @@ class Sentinel2SR(Sentinel2):
         Mask.fromBand('SCL', scl,
                       negatives=['cloud', 'saturated', 'dark', 'shadow', 'water',
                                  'clouds_mid_probability', 'clouds_high_probability',
-                                 'cirrus', 'snow', 'unclassified']),
+                                 'cirrus', 'snow']),
         Mask.fromBand('QA60', Sentinel2.qa60)
     )
 
@@ -273,16 +322,29 @@ class Sentinel1(ImageCollection):
         vv = filt('VV')
         vh = filt('VH')
 
-        filters = {
-            'HH': ee.Filter.And(hh, hv.Not(), vv.Not(), vh.Not()),
-            'HH-HV': ee.Filter.And(hh, hv, vv.Not(), vh.Not()),
-            'VV': ee.Filter.And(hh.Not(), hv.Not(), vv, vh.Not()),
-            'VV-VH': ee.Filter.And(hh.Not(), hv.Not(), vv, vh)
-        }
-        if polarisation not in filters:
-            raise ValueError('Polarisation {} not available, options are: {}'.format(polarisation, filters.keys()))
+        if isinstance(polarisation, str):
+            filters = {
+                'HH': ee.Filter.And(hh, hv.Not(), vv.Not(), vh.Not()),
+                'HH-HV': ee.Filter.And(hh, hv, vv.Not(), vh.Not()),
+                'VV': ee.Filter.And(hh.Not(), hv.Not(), vv, vh.Not()),
+                'VV-VH': ee.Filter.And(hh.Not(), hv.Not(), vv, vh)
+            }
+            if polarisation not in filters:
+                raise ValueError('Polarisation {} not available, options are: {}'.format(polarisation, filters.keys()))
+
+            filter = filters[polarisation]
+        else:
+            if list(polarisation) == ['HH']:
+                filter = ee.Filter.And(hh, hv.Not(), vv.Not(), vh.Not())
+            elif list(polarisation) == ['HH', 'HV'] or list(polarisation) == ['HV', 'HH']:
+                filter = ee.Filter.And(hh, hv, vv.Not(), vh.Not())
+            elif list(polarisation) == ['VV']:
+                filter = ee.Filter.And(hh.Not(), hv.Not(), vv, vh.Not())
+            else:
+                filter = ee.Filter.And(hh.Not(), hv.Not(), vv, vh)
+
+        return col.filter(filter)
 
-        return col.filter(filters[polarisation])
 
     def filter_orbit(self, ascending=True, collection=None):
         """ Filter by orbit ascending or descending """
@@ -307,9 +369,15 @@ class Sentinel1(ImageCollection):
 
     def convert_to_intensity(self, image):
         """ Convert values as they come in GEE (db) to intensity (DN) """
-        angle = image.select('angle')
-        intensity = ee.Image().expression('10**(v/10)', dict(v=image))
-        return intensity.addBands(angle, overwrite=True)
+        rest = image.bandNames().remove('angle')
+        intensity = ee.Image().expression('10**(v/10)', dict(v=image.select(rest)))
+        return image.addBands(intensity, overwrite=True)
+
+    def convert_to_db(self, image):
+        """ Convert values from intensity to db """
+        rest = image.bandNames().remove('angle')
+        intensity = ee.Image().expression('log10(v)*10', dict(v=image.select(rest)))
+        return image.addBands(intensity, overwrite=True)
 
     @staticmethod
     def _lee_filter(image):
@@ -321,8 +389,8 @@ class Sentinel1(ImageCollection):
 
         # image must be in natural units, i.e. not in dB!
         # Set up 3x3 kernels
-        weights3 = ee.List.repeat(ee.List.repeat(1,3), 3)
-        kernel3 = ee.Kernel.fixed(3,3, weights3, 1, 1, False)
+        weights3 = ee.List.repeat(ee.List.repeat(1, 3), 3)
+        kernel3 = ee.Kernel.fixed(3, 3, weights3, 1, 1, False)
 
         mean3 = image.reduceNeighborhood(ee.Reducer.mean(), kernel3)
         variance3 = image.reduceNeighborhood(ee.Reducer.variance(), kernel3)
@@ -408,10 +476,10 @@ class Sentinel1(ImageCollection):
 
         # and add the bands for rotated kernels
         for i in range(1, 4):
-            dir_mean = dir_mean.addBands(image.reduceNeighborhood(ee.Reducer.mean(), rect_kernel.rotate(i)).updateMask(directions.eq(2 * i + 1)));
-            dir_var= dir_var.addBands(image.reduceNeighborhood(ee.Reducer.variance(), rect_kernel.rotate(i)).updateMask(directions.eq(2 * i + 1)));
-            dir_mean = dir_mean.addBands(image.reduceNeighborhood(ee.Reducer.mean(), diag_kernel.rotate(i)).updateMask(directions.eq(2 * i + 2)));
-            dir_var= dir_var.addBands(image.reduceNeighborhood(ee.Reducer.variance(), diag_kernel.rotate(i)).updateMask(directions.eq(2 * i + 2)));
+            dir_mean = dir_mean.addBands(image.reduceNeighborhood(ee.Reducer.mean(), rect_kernel.rotate(i)).updateMask(directions.eq(2 * i + 1)))
+            dir_var= dir_var.addBands(image.reduceNeighborhood(ee.Reducer.variance(), rect_kernel.rotate(i)).updateMask(directions.eq(2 * i + 1)))
+            dir_mean = dir_mean.addBands(image.reduceNeighborhood(ee.Reducer.mean(), diag_kernel.rotate(i)).updateMask(directions.eq(2 * i + 2)))
+            dir_var= dir_var.addBands(image.reduceNeighborhood(ee.Reducer.variance(), diag_kernel.rotate(i)).updateMask(directions.eq(2 * i + 2)))
 
 
         # "collapse" the stack into a single band image (due to masking, each pixel has just one value in it's directional band, and is otherwise masked)
@@ -434,6 +502,9 @@ class Sentinel1(ImageCollection):
 
         https://groups.google.com/g/google-earth-engine-developers/c/puhSXaJ7NmI/m/TNOd0knyBQAJ
         """
+        # convert to natural units
+        image = self.convert_to_intensity(image)
+
         angleB = self.getBandByName('angle')
         angle = angleB.alias if renamed else angleB.name
         bands = image.bandNames()
@@ -447,4 +518,5 @@ class Sentinel1(ImageCollection):
             return im.addBands(f, overwrite=True)
 
         filtered = ee.Image(bands.iterate(wrap, image))
-        return filtered
+        db = self.convert_to_db(filtered)
+        return db
